//
//  AppDelegate.m
//  DDCCTL
//
//  Created by Rens Breur on 15/01/15.
//  Copyright (c) 2015 Rens Breur. All rights reserved.
//

#import "AppDelegate.h"
#import "MenuItemController.h"
#import "HUDController.h"
#import "HotkeyController.h"

#define kHotkeyVolumeUp 111
#define kHotkeyVolumeDown 103
#define kHotkeyVolumeMute 109

@interface AppDelegate ()

// Helper classes
@property (nonatomic, strong) MenuItemController *menuItemController;
@property (nonatomic, strong) HUDController *hudController;
@property (nonatomic, strong) HotkeyController *hotkeyController;

@end

@implementation AppDelegate

- (void)applicationDidFinishLaunching:(NSNotification *)aNotification
{
    // Setup
    self.ddcManager = [DDCManager sharedManager];
    [self setupMenuItem];
    [self setupHUDPanel];
    [self setupHotkeys];
}

#pragma mark - Menu Item

- (void)setupMenuItem
{
    // Setup menu item
    self.menuItemController = [[MenuItemController alloc] init];
    __weak AppDelegate *appDelegate = self;
    self.menuItemController.menuClicked = ^{
        [appDelegate showHUDPanel];
    };
    [self updateMenuItem];
}

- (void)updateMenuItem
{
    // Set image
    int imageNumber = ceil(self.ddcManager.volume*3.0);
    NSString *imageName = [NSString stringWithFormat:@"volume%d", imageNumber];
    [self.menuItemController setImage:imageName];
}

#pragma mark - HUD

- (void)setupHUDPanel
{
    // Create HUDPanel instance
    self.hudController = [[HUDController alloc] init];
}

- (void)showHUDPanel
{
    // Show HUD Panel
    self.hudController.value = self.ddcManager.volume;
    [self.hudController showHUD];
}

#pragma mark - Hotkeys

- (void)setupHotkeys
{
    // Setup
    self.hotkeyController = [[HotkeyController alloc] init];
    __weak AppDelegate *appDelegate = self;
    
    // Register F10 (Mute)
    [self.hotkeyController addListener:^(HotkeyEvent event){
        [appDelegate hotkeyChangeVolume:0];
    } forHotkey:kHotkeyVolumeMute];
    
    // Register F11 (Volume down)
    [self.hotkeyController addListener:^(HotkeyEvent event){
        [appDelegate hotkeyChangeVolume:appDelegate.ddcManager.volume - (event == HotkeyEventKeyPressed ? (0.0625 ) : 0.0625)];
    } forHotkey:kHotkeyVolumeDown];
    
    // Register F12 (Volume up)
    [self.hotkeyController addListener:^(HotkeyEvent event){
        [appDelegate hotkeyChangeVolume:appDelegate.ddcManager.volume + (event == HotkeyEventKeyPressed ?  (0.0625) : 0.0625)];
    } forHotkey:kHotkeyVolumeUp];
}

- (void)hotkeyChangeVolume:(float)volume
{
    // Update the volume
    self.ddcManager.volume = volume;
    [self updateMenuItem];
    [self showHUDPanel];
}



@end
