//
//  DDCManager.m
//  DDCCTL
//
//  Created by Rens Breur on 15/01/15.
//  Copyright (c) 2015 Rens Breur. All rights reserved.
//

#import "DDCManager.h"
#include "DDC.h"

@interface DDCManager ()

@property (nonatomic) unsigned int displayIdentifier;
@property (nonatomic) int maxVolume;

@end

@implementation DDCManager

DDCManager *ddcManager;

+ (DDCManager *)sharedManager {
    if (!ddcManager) {
        ddcManager = [[DDCManager alloc] init];
        [ddcManager setup];
    }
    return ddcManager;
}

#pragma mark - Setup

- (void)setup {
    [self readDisplayIdentifier];
    [self readVolume];
}

- (void)readDisplayIdentifier {
    NSScreen *display = NSScreen.screens.lastObject;
    self.displayIdentifier =  [[[display deviceDescription] objectForKey:@"NSScreenNumber"] unsignedIntValue];
}

#pragma mark - Volume

- (void)setVolume:(float)volume {
    if (volume < 0.00) _volume = 0.00;
    else if (volume > 1.00) _volume = 1.00;
    else _volume = volume;
    [self writeVolume];
}

#pragma mark - Commands

- (void)writeVolume
{
    // Send write command
    struct DDCWriteCommand command = {AUDIO_SPEAKER_VOLUME, (int)(self.volume * self.maxVolume)};
    DDCWrite(self.displayIdentifier, &command);
}

- (void)readVolume
{
    // Send read command
    struct DDCReadCommand command = {AUDIO_SPEAKER_VOLUME};
    DDCRead(self.displayIdentifier, &command);
    self.maxVolume = command.max_value;
    self.volume = command.current_value / (float)self.maxVolume;
}

@end
