//
//  HUDController.m
//  DDCCTL
//
//  Created by Rens Breur on 17/01/15.
//  Copyright (c) 2015 Rens Breur. All rights reserved.
//

#import "HUDController.h"
#import <QuartzCore/QuartzCore.h>
#import "DDCCTL-Swift.h"
#define kHUDWidth 200
#define kHUDHeight 200
#define kHUDMargins 60
#define kHUDOpacity 1
#define kHUDImageTopOffset 12
#define kHUDProgressMargins 20
#define kHUDProgressWitdh 161

#define kHUDProgressHeight 8

#define kOnImage [NSImage imageNamed:@"audio_slider"]
#define kOffImage [NSImage imageNamed:@"audio_slider_off"]


@interface HUDController (){
    VolumeProgressView *progress;
    NSImageView *imageView;
}

@property (nonatomic, strong) NSWindow *window;
@property (nonatomic, weak) NSSlider *slider;

@property (nonatomic, strong) NSTimer *inactivityTimer;

@end

@implementation HUDController

- (instancetype)init {
    if (self = [super init]) {
        [self setupWindow];
    }
    return self;
}

- (void)showHUD
{
    [self showWindow];
    [self.inactivityTimer invalidate];
    self.inactivityTimer = [NSTimer scheduledTimerWithTimeInterval:1.5 target:self selector:@selector(closeHUD) userInfo:nil repeats:NO];
}

- (void)closeHUD
{
    [[self.window animator] setAlphaValue:0];
}

- (void)setValue:(float)value {
    _value = value;
    progress.currentValue = value;
    [self.slider setFloatValue:value];
    if (value == 0){
        imageView.image = kOffImage;
        return;
    }
    imageView.image = kOnImage;

}

#pragma mark - Window
- (void)setupWindow
{
    // Setup window
    NSScreen *screen = [NSScreen mainScreen];
    CGRect screenFrame = [screen frame];
    NSRect contentSize = NSMakeRect(screenFrame.size.width / 2 - kHUDWidth / 2, kHUDHeight - kHUDMargins, kHUDWidth, kHUDHeight);
    self.window = [[NSWindow alloc] initWithContentRect:contentSize styleMask:NSBorderlessWindowMask backing:NSBackingStoreBuffered defer:YES];
    self.window.backgroundColor = [NSColor whiteColor];
    self.window.title = @"DDCCTL";
    [self.window setMovableByWindowBackground:true];
    [self.window setLevel:NSStatusWindowLevel];
    self.window.opaque = NO;
    self.window.backgroundColor = [NSColor clearColor];
    self.window.alphaValue = 0;
    self.window.ignoresMouseEvents = YES;
    
    // Setup view
    NSVisualEffectView *view = [[NSVisualEffectView alloc] initWithFrame:contentSize];
    [view setBlendingMode:NSVisualEffectBlendingModeBehindWindow];
    view.layer.cornerRadius = 10;
    [view setState:NSVisualEffectStateActive];
    [view setMaterial:NSVisualEffectMaterialLight];
    [view setMaskImage:[self maskImageWithBounds:view.bounds]];
    
    // Add progressbar
    progress = [[VolumeProgressView alloc] initWithFrame:CGRectMake(kHUDProgressMargins, kHUDProgressMargins, kHUDProgressWitdh, kHUDProgressHeight)];

    NSSlider *slider = [[NSSlider alloc] initWithFrame:CGRectMake(18, 18, 284, 21)];
    slider.minValue = 0.00;
    slider.maxValue = 1.00;
    slider.floatValue = self.value;
    [view addSubview:slider];
    self.slider = slider;
    [slider setHidden:true];
    
    [view addSubview:progress];
    imageView = [[NSImageView alloc] initWithFrame:NSRectFromCGRect(CGRectMake(kHUDHeight / 2 - kOffImage.size.width / 2, (kHUDHeight / 2  - kOffImage.size.height / 2)  + kHUDImageTopOffset, kOffImage.size.width, kOffImage.size.height))];
    // Show window
    imageView.image = kOnImage;
    
    [view addSubview:imageView];
    [self.window setContentView:view];
    
}

- (void)showWindow {
    [self.window makeKeyAndOrderFront:self];
    [self.window.animator setAlphaValue:kHUDOpacity];
}

- (CGPathRef) newPathForRoundedRect:(CGRect)rect radius:(CGFloat)radius {
    CGMutablePathRef retPath = CGPathCreateMutable();
    CGRect innerRect = CGRectInset(rect, radius, radius);
    CGFloat inside_right = innerRect.origin.x + innerRect.size.width;
    CGFloat outside_right = rect.origin.x + rect.size.width;
    CGFloat inside_bottom = innerRect.origin.y + innerRect.size.height;
    CGFloat outside_bottom = rect.origin.y + rect.size.height;
    CGFloat inside_top = innerRect.origin.y;
    CGFloat outside_top = rect.origin.y;
    CGFloat outside_left = rect.origin.x;
    CGPathMoveToPoint(retPath, NULL, innerRect.origin.x, outside_top);
    CGPathAddLineToPoint(retPath, NULL, inside_right, outside_top);
    CGPathAddArcToPoint(retPath, NULL, outside_right, outside_top, outside_right, inside_top, radius);
    CGPathAddLineToPoint(retPath, NULL, outside_right, inside_bottom);
    CGPathAddArcToPoint(retPath, NULL,  outside_right, outside_bottom, inside_right, outside_bottom, radius);
    CGPathAddLineToPoint(retPath, NULL, innerRect.origin.x, outside_bottom);
    CGPathAddArcToPoint(retPath, NULL,  outside_left, outside_bottom, outside_left, inside_bottom, radius);
    CGPathAddLineToPoint(retPath, NULL, outside_left, inside_top);
    CGPathAddArcToPoint(retPath, NULL,  outside_left, outside_top, innerRect.origin.x, outside_top, radius);
    CGPathCloseSubpath(retPath);
    return retPath;
}


- (NSImage *) maskImageWithBounds: (NSRect) bounds
{
    return [NSImage imageWithSize:bounds.size flipped:YES drawingHandler:^BOOL(NSRect dstRect) {
        
        NSBezierPath *path = [NSBezierPath bezierPathWithRoundedRect:bounds xRadius:20.0 yRadius:20.0];
        
        [path setLineJoinStyle:NSRoundLineJoinStyle];
        [path fill];
        
        return YES;
    }];
}



@end
