//
//  HotkeyController.m
//  DDCCTL
//
//  Created by Rens Breur on 17/01/15.
//  Copyright (c) 2015 Rens Breur. All rights reserved.
//

#import "HotkeyController.h"
#import <Carbon/Carbon.h>

@interface HotkeyController ()

@property (nonatomic, strong) NSMutableDictionary *hotkeyFunctions;

@property (nonatomic, strong) NSTimer *hotkeyDelay;
@property (nonatomic, strong) NSTimer *hotkeyTimer;

@property (nonatomic) int activeEvent;
@property (nonatomic) HotkeyEvent activeEventType;

@end

@implementation HotkeyController

HotkeyController *hkC;

-(instancetype)init {
    if (self = [super init]) {
        self.hotkeyFunctions = [[NSMutableDictionary alloc] init];
        hkC = self;
    }
    return self;
}

- (void)addListener:(void (^)(HotkeyEvent))block forHotkey:(int)hotKey {
    [self.hotkeyFunctions setObject:block forKey:[NSNumber numberWithInt:hotKey]];
    [self registerHotkey:hotKey withSignature:hotKey andId:hotKey];
}

#pragma mark - Carbon hotkey register

OSStatus myHotKeyHandler(EventHandlerCallRef nextHandler, EventRef anEvent, void *userData);

- (void)registerHotkey:(int)hotKey withSignature:(OSType)signature andId:(int)identifier
{
    // Setup handler
    EventHotKeyRef myHotKeyRef;
    EventHotKeyID myHotKeyID;
    EventTypeSpec eventTypeHotkeyPress;
    eventTypeHotkeyPress.eventClass=kEventClassKeyboard;
    eventTypeHotkeyPress.eventKind= kEventHotKeyPressed;
    EventTypeSpec eventTypeHotkeyRelease;
    eventTypeHotkeyRelease.eventClass=kEventClassKeyboard;
    eventTypeHotkeyRelease.eventKind= kEventHotKeyReleased;
    EventTypeSpec events[2] = {eventTypeHotkeyPress, eventTypeHotkeyRelease};
    EventTypeSpec *eventsPointer = events;
    InstallApplicationEventHandler(&myHotKeyHandler,2,eventsPointer,NULL,NULL);

    // Register key
    myHotKeyID.signature = signature;
    myHotKeyID.id = identifier;
    RegisterEventHotKey(hotKey, 0, myHotKeyID, GetApplicationEventTarget(), 0, &myHotKeyRef);
}

OSStatus myHotKeyHandler(EventHandlerCallRef nextHandler, EventRef anEvent, void *userData) {
    EventHotKeyID hkRef;
    GetEventParameter(anEvent,kEventParamDirectObject,typeEventHotKeyID,NULL,sizeof(hkRef),NULL,&hkRef);
    EventKind eventKind = GetEventKind(anEvent);
    if (eventKind == kEventHotKeyPressed)
    {
        // Hotkey pressed
        hkC.activeEvent = hkRef.id;
        [hkC hotkeyDown];
        if (!hkC.hotkeyDelay && !hkC.hotkeyTimer) {
            hkC.hotkeyDelay = [NSTimer scheduledTimerWithTimeInterval:0.4 target:hkC selector:@selector(hotkeyUpTimerStart) userInfo:nil repeats:NO];
        }
    }
    else if (eventKind == kEventHotKeyReleased)
    {
        // Hotkey released
        [hkC.hotkeyDelay invalidate];
        hkC.hotkeyDelay = nil;
        [hkC.hotkeyTimer invalidate];
        hkC.hotkeyTimer = nil;
    }
    return noErr;
}

#pragma mark - Hotkey timers

- (void)hotkeyUpTimerStart {
    self.hotkeyTimer = [NSTimer scheduledTimerWithTimeInterval:0.1 target:self selector:@selector(hotkeyPressed) userInfo:nil repeats:YES];
}

- (void)hotkeyDown {
    void(^hotkeyAction)(HotkeyEvent) = [self.hotkeyFunctions objectForKey:[NSNumber numberWithInt:self.activeEvent]];
    hotkeyAction(HotkeyEventKeyDown);
}

- (void)hotkeyPressed {
    void(^hotkeyAction)(HotkeyEvent) = [self.hotkeyFunctions objectForKey:[NSNumber numberWithInt:self.activeEvent]];
    hotkeyAction(HotkeyEventKeyPressed);
}

@end
